package
{
    import flash.events.MouseEvent;
    import flash.events.TextEvent;
    import flash.utils.Dictionary;
    
    import mx.collections.ArrayCollection;
    import mx.controls.List;
    import mx.controls.dataGridClasses.DataGridItemRenderer;
    
    /**
     * Logic behind the RegExpTester class.
     */
    public class RegExpHandler
    {
        [Bindable]
        public var regExpTester:RegExpTester;
        
        // Map between checkbox names and the flag it sets
        private var _flags:Array = [["global", "g"],
                                    ["ignoreCase", "i"],
                                    ["dotall", "s"],
                                    ["multiline", "m"],
                                    ["extended", "x"]];
        
        /**
         * Default constructor
         */
        public function RegExpHandler()
        {
        }

        /**
         * After a user action determine if we should automatically
         * rerun the current search or replace operation.
         */
        public function checkExecute():void
        {
            if (regExpTester.continuous.selected)
            {
                execute();
            }
        }
        
        /**
         * Run the current search or replace operation.
         */
        public function execute():void
        {
            if (regExpTester.mode.selectedItem.state == "replaceMode")
            {
                replace();
            }
            else if (regExpTester.mode.selectedItem.state == "splitMode")
            {
                split();
            }
            else
            {
                search();
            }
        }
        
        /**
         * Handle the user clicking on a row in the data grid and highlight
         * that match in the text input window.
         * @param mouseEvent Event data
         */
        public function highlight(mouseEvent:MouseEvent):void
        {
            if (!("data" in mouseEvent.target))
            {
                return;
            }
            var data:Object = mouseEvent.target.data;
            if ((data is Entry) && (data.start != -1))
            {
                regExpTester.input.setSelection(data.start, data.end);
                regExpTester.input.setFocus();
            }
        }
        
        /**
         * Extract the text that the user is performing the search or replace against.
         * @return User entered text
         */
        private function getText():String
        {
            return regExpTester.input.text;
        }

        /**
         * Set the input text. This translates it into html format
         * so that it will display correctly.
         * @param text Text to set
         */
        private function setText(text:String):void
        {
            var formattedText:String = text.replace(/[\r\n]/g, "<br/>");
            regExpTester.input.htmlText = formattedText;
        }
        
        /**
         * Loop over the available flags and determine which are set.
         * @return String of flags the user has checked
         */
        private function determineFlags():String
        {
            var flags:String;
            for each (var flag:Array in _flags)
            {
                if (regExpTester[flag[0]].selected)
                {
                    flags += flag[1];
                }
            }
            return flags;
        }
        
        /**
         * Create the regular expression to execute against. If the
         * regular expression is blank this will return null.
         * @return Regular expression
         * 
         */
        private function getRegExp(allowEmpty:Boolean = false):RegExp
        {
            var regExpText:String = regExpTester.regexp.text;
            if ((regExpText == null) || (regExpTester.regexp.text == ""))
            {
                if (allowEmpty)
                {
                    return new RegExp("", determineFlags());    
                }
                else
                {
                    return null;
                }
            }
            return new RegExp(regExpText, determineFlags());
        }

        /**
         * Perform a text replace operation.
         */
        private function replace():void
        {
            var regExp:RegExp = getRegExp(true);
            var rawText:String = getText();

            // remove any formatting that may have been there from the last search operation
            setText(rawText);
            regExpTester.replaceOutput.text = "Replacing " + regExp.toString() + " with " + regExpTester.replaceText.text + "\n";
            regExpTester.replaceOutput.text += rawText.replace(regExp, regExpTester.replaceText.text);
        }

        /**
         * Perform a text split operation.
         */
        private function split():void
        {
            var regExp:RegExp = getRegExp(true);
            var rawText:String = getText();

            var results:ArrayCollection = new ArrayCollection();

            var entry:Entry =  new Entry();
            entry.match = "Splitting with: " + regExp.toString();
            results.addItem(entry);

            var split:Array = rawText.split(regExp);
            for (var i:int = 0; i < split.length; i++)
            {
                entry = new Entry();
                entry.position = String(i);
                entry.match = split[i];
                results.addItem(entry);
            }

            // remove any formatting that may have been there from the last search operation
            setText(rawText);
            regExpTester.splitData.dataProvider = results;            
        }
        
        /**
         * Perform a text search operation.
         */
        private function search():void
        {
            var regExp:RegExp = getRegExp();
            var rawText:String = getText();

            if (regExp == null)
            {
                setText(rawText);
                regExpTester.results.dataProvider = new ArrayCollection();
                return;    
            }
            
            var results:ArrayCollection = new ArrayCollection();
            
            var entry:Entry =  new Entry();
            entry.match = "Searching with: " + regExp.toString();
            results.addItem(entry);

            var result:Object = regExp.exec(rawText);
            if (result == null)
            {
                // no matches found
                setText(rawText);
            }    
            else
            {
                var formattedText:String = "";
                var lastIndex:int = 0;
                var doLoop:Boolean = regExpTester.global.selected;
                do
                {
                    for (var i:int = 0; i < result.length; i++)
                    {
                        entry = new Entry();
                        if (i == 0)
                        {
                            entry.position = result.index;
                            entry.start = result.index;
                            entry.end = result.index + result[0].length;
                        }
                        else
                        {
                            entry.group = i;
                        }
                        if (result[i] != null)
                        {
                            entry.match = result[i].replace(/\n/g, "\\n").replace(/\r/g, "\\r").replace(/\t/g, "\\t");
                        }
                        results.addItem(entry);
                    }
                    formattedText += rawText.substring(lastIndex, result.index);
                    // TODO: Handle this via a style
                    formattedText += "<font color=\"#FF0000\"><b>";
                    formattedText += rawText.substr(result.index, result[0].length);
                    formattedText += "</b></font>";
                    lastIndex = result.index + result[0].length;
                    result = regExp.exec(rawText);
                } while ((doLoop) && (result != null));
                
                formattedText += rawText.substring(lastIndex, rawText.length);
                setText(formattedText);
            }
            regExpTester.results.dataProvider = results;
        }
    }
}

/**
 * Fields for a row in the data grid.
 */
class Entry
{
    public var position:String;
    public var group:String;
    public var match:String;
    public var start:int = -1;
    public var end:int = -1;
    
    public function Entry()
    {
    }
}
